function [e, Tc, pc, wt, dh_to_sat] = moist_ape_thermo(T, p, r, thermo_type)

% Consistent thermodynamics for computation of moist ape (Lorenz 1979, Randall and Wang 1992, Wang and Randall 1994)

% Approach used is to specify es, which gives L, and then specify cpw which
% together with L implies c (Lorenz 79 equation 10)

% The 'full' formulation includes the effects of ice following Wang and Randall, but with es given by a modified Teten's formula

% Variable key:
% s entropy
% h enthalpy
% w mixing ratio
% L latent heat of evaporation or sublimation
% e vapor pressure
% T temperature
% p pressure
% wt total mixing ratio
% r generalized relative humidity (a ratio of mixing ratios when
% above one, and a ratio of vapor pressures when below one)
  
  error(nargchk(4, 4, nargin))     

  switch lower(thermo_type)
   case 'full'
    cpw            = PARS('cp_v'); % specific heat capacity of water vapor
   case 'simple'
    % assume have the same heat capacity for water vapor and dry air in the simple case
    cpw            = PARS('cp');
   otherwise
    error(['Unknown type, ', thermo_type, 'of thermodynamic formulation.'])
  end


  R              = PARS('gas_constant');
  Rw             = PARS('gas_constant_v');
  cp             = PARS('cp');
  eps            = R/Rw;

  use_bolton_Tc = 0;

  % latent heat and saturation vapor pressure
  [es, L] = moist_ape_es(T, thermo_type);


  % mixing ratios
  rmin = min(r, 1);
  rmax = max(r, 1);
  e    = es.*rmin;
  w    = eps*e./(p-e);
  wt   = w.*rmax;


  % condensation temperature
  % equation 15 Lorenz with wt=w, noting typo pointed out by Randall 1992 
  % use finite difference to evaluate ds/dT so can use arbitrary es
  % and its related L
  s0 = ((cp+wt.*cpw).*log(T)-R*log(p-e)-wt.*Rw.*log(e))./(1+wt);

 
  if use_bolton_Tc
   disp('using Bolton Tc')
   Tc = bolton_temp_lcl(e,T);
  else

   % iterate
   dT    = 1; % initial temperature step
   Tc    = T; % initial guess
   T_perturb = 0.1; % used to evaluated ds_dT
   while any(abs(dT)>0.001)
    es_at_Tc = moist_ape_es(Tc, thermo_type);
    pc       = es_at_Tc.*(eps+wt)./wt; % equation 6 lorenz
    s        = moist_ape_sat_entropy(Tc, pc, wt, thermo_type);

    Tc_perturb       = Tc+T_perturb;

    es_at_Tc_perturb = moist_ape_es(Tc_perturb, thermo_type);
    pc_perturb       = es_at_Tc_perturb.*(eps+wt)./wt; 
    s_perturb        = moist_ape_sat_entropy(Tc_perturb, pc_perturb, wt, thermo_type);

    ds_dT     =  (s_perturb-s)/T_perturb;
    dT        = (s0-s)./ds_dT;
    Tc        = Tc + dT;
   end
  end


  if any((Tc-T)>0)
   error('Tc unphysical')
  end


  % condensation pressure
  [es_at_Tc, L_at_Tc] = moist_ape_es(Tc, thermo_type);
  pc                  = es_at_Tc.*(eps+w)./w; % equation 6 lorenz


  % change in enthalpy by bringing to saturation
  dh_to_sat = (cp+wt.*cpw).*(Tc-T)./(1+wt); % equation 12 Lorenz




